@@ -5,8 +5,8 @@
 # Author          : Johan Vromans
 # Created On      : Tue Sep  6 16:09:10 2016
 # Last Modified By: Johan Vromans
-# Last Modified On: Sat Oct  8 19:18:51 2016
-# Update Count    : 46
+# Last Modified On: Tue Feb 28 13:51:29 2017
+# Update Count    : 68
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -18,22 +18,24 @@ use utf8;
 
 package Data::iRealPro::Input;
 
-our $VERSION = "0.05";
+our $VERSION = "1.03";
 
 use Data::iRealPro::URI;
 use Data::iRealPro::Input::Text;
 use Encode qw ( decode_utf8 );
+use File::Basename ();
 
 sub new {
     my ( $pkg, $options ) = @_;
 
     my $self = bless( { variant => "irealpro" }, $pkg );
 
-    for ( qw( trace debug verbose output variant transpose select ) ) {
+    for ( qw( trace debug verbose output variant transpose
+	      neatify select ) ) {
 	$self->{$_} = $options->{$_} if exists $options->{$_};
     }
 
-    for ( qw( playlist ) ) {
+    for ( qw( playlist catalog ) ) {
 	$self->{$_} = decode_utf8($options->{$_}) if exists $options->{$_};
     }
 
@@ -55,6 +57,11 @@ sub parsefiles {
 
     my $all;
     foreach my $file ( @files ) {
+	warn("Parsing: $file...\n") if $self->{verbose};
+	unless ( $self->{playlist} ) {
+	    my @p = File::Basename::fileparse( $file,  qr/\.[^.]*/ );
+	    $self->{playlist} = $p[0];
+	}
 	my $u = $self->parsefile($file);
 	unless ( $all ) {
 	    $all = $u;
@@ -89,7 +96,11 @@ sub parsedata {
 
     else {
 	if ( $data =~ /^Song( \d+)?:/ ) {
-	    $all = Data::iRealPro::Input::Text->encode($data);
+	    $all = Data::iRealPro::Input::Text::encode( $self, $data );
+	}
+	elsif ( $data =~ /^(?:<\?xml.*?>\s*)?<!doctype\s+score-partwise\s+/i ) {
+	    require Data::iRealPro::Input::MusicXML;
+	    $all = Data::iRealPro::Input::MusicXML::encode( $self, $data );
 	}
 	else {
 	    # Extract URL.
