@@ -5,8 +5,8 @@
 # Author          : Johan Vromans
 # Created On      : Fri Jan 15 19:15:00 2016
 # Last Modified By: Johan Vromans
-# Last Modified On: Tue Nov 15 16:15:56 2016
-# Update Count    : 1457
+# Last Modified On: Thu Dec 15 21:46:53 2016
+# Update Count    : 1512
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -22,13 +22,13 @@ package Data::iRealPro::Output::Imager;
 
 use parent qw( Data::iRealPro::Output::Base );
 
-our $VERSION = "0.12";
+our $VERSION = "1.00";
 
 use Data::Dumper;
 use Text::CSV_XS;
 use Encode qw( encode_utf8 );
 
-use constant FONTSX => 0;
+use constant FONTSX => 1;
 
 sub new {
     my ( $pkg, $options ) = @_;
@@ -164,7 +164,8 @@ sub process {
 
     if ( $outtype eq "pdf" && eval { require PDF::API2 } ) {
 	$self->{pdf} = PDF::API2->new;
-	$self->{pdf}->mediabox( 0, PAGE_HEIGHT, PAGE_WIDTH, 0 );
+#	$self->{pdf}->mediabox( 0, PAGE_HEIGHT, PAGE_WIDTH, 0 );
+	$self->{pdf}->mediabox( 0, 0, PAGE_WIDTH, PAGE_HEIGHT );
     }
     elsif ( $outtype =~ /^png|jpg$/ && eval { require Imager } ) {
 	# ok
@@ -325,7 +326,7 @@ sub make_image {
 	# Calculate the required heigth and create the canvas.
 	my $v = $tm;		# top margin
 	$v += int( ( @$cells + 15 ) / $numcols ) * $dy; # cells
-	$v += ( $cells->[-1]->vs || 0 ) * 121;	       # extra
+	$v += $cells->[-1]->vs * 121;	       # extra
 	$v = CANVAS_HEIGHT if $v < CANVAS_HEIGHT; # minimal
 	$self->{im} = Imager->new( xsize => CANVAS_WIDTH,
 				   ysize => $v,
@@ -368,6 +369,7 @@ sub make_image {
     if ( $dy < 1.6*$musicsize ) {
 	$dy = 1.6*$musicsize;
     }
+    my %missing_glyphs;
 
     $self->{pages} = 0;
 
@@ -408,6 +410,11 @@ sub make_image {
 
     my $low;			# water mark to crop image
 
+    # Discard final (trailing?) empty cells for a correct $low.
+    # Note: Empty cells have just a 'vs' element.
+    pop( @$cells )
+      while $cells->[-1] && keys( %{ $cells->[-1] } ) == 1;
+
     # Process the cells.
     for ( my $i = 0; $i < @$cells; $i++ ) {
 	# {{{
@@ -470,7 +477,7 @@ sub make_image {
 	    if ( /^repeatRight$/ ) {
 		$col = $red;
 		if ( ($i+1) % $numcols
-		     && $i < @$cells
+		     && $i < @$cells-1
 		     && $cells->[$i+1]->lbar
 		     && $cells->[$i+1]->lbar eq "repeatLeft" ) {
 		    $cells->[$i+1]->lbar = "repeatRightLeft";
@@ -526,15 +533,29 @@ sub make_image {
 	    next unless $_;
 	    my ( $disp, $t ) = @$_;
 
-	    if ( FONTSX && $self->{pdf} ) {
-		for ( split( //, $t ) ) {
-		    next if $textfont->uniByEnc(ord($_));
-		    warn( sprintf( "Missing glyph U+%04X\n", ord($_) ) );
+	    if ( FONTSX ) {
+		if ( $self->{pdf} ) {
+		    # $textfont->{' font'} = Font::TTF::Font
+		    for ( split( //, $t ) ) {
+			next if $textfont->uniByEnc(ord($_));
+			my $c = ord(substr($t,$i,1));
+			next if $missing_glyphs{$c};
+			$missing_glyphs{$c} = 1;
+		    }
+		}
+		if ( $self->{im} ) {
+		    my @c = $textfont->has_chars( string => $t );
+		    for ( my $i = 0; $i < @c; $i++ ) {
+			next if $c[$i];
+			my $c = ord(substr($t,$i,1));
+			next if $missing_glyphs{$c};
+			$missing_glyphs{$c} = 1;
+		    }
 		}
 	    }
 
 	    my $hack;
-	    # Sometimes, THAI PAIYANNOI (U+2e7) is abused as
+	    # Sometimes, THAI PAIYANNOI (U+e2f) is abused as
 	    # MUSICAL SYMBOL EIGHTH REST (u+1d13e).
 	    $t =~ s/\x{e2f}/\x{1d13e}/g && $hack++;
 	    # Likewise CYRILLIC SMALL LETTER GHE WITH UPTURN (U+491)
@@ -562,54 +583,78 @@ sub make_image {
 	    my $c = $_;
 	    my $font = $cell->sz ? $chrdfont : $chordfont;
 
-	    if ( $c =~ /^repeat1Bar$/ ) {
+	    if ( $c =~ /repeat\dBars?/ ) {
 
-		# Find previous bar line.
-		my $pb = $i;
-		while ( $pb >= 0) {
-		    last if $cells->[$pb]->lbar
-		      || ( $pb > 1 && $cells->[$pb-1]->rbar );
-		    $pb--;
+		# npp_strict
+		#   both go at the border of this and the next cell
+		# else
+		#   1bar and 2bar not before barline: center in measure
+		#   2bar before barline: center on barline
+
+		if ( $self->{npp_strict} ) {
+		    $x += $dx;
+		    if (  $c eq "repeat1Bar" ) {
+			$self->npp_repeat( $x, $y, 1 );
+		    }
+		    else {
+			$self->npp_repeat( $x, $y, 2 );
+		    }
 		}
-		# Find next bar line.
-		my $nb = $i;
-		while ( $nb < @$cells ) {
-		    last if $cells->[$nb]->rbar
-		      || ( $nb+1 < @$cells && $cells->[$nb+1]->lbar );
-		    $nb++;
+		elsif ( $c eq "repeat1Bar"
+		     || ( $c eq "repeat2Bars" && !$cell->rbar )
+		   ) {
+
+		    # Find previous bar line.
+		    my $pb = $i;
+		    while ( $pb >= 0) {
+			last if $cells->[$pb]->lbar
+			  || ( $pb > 1 && $cells->[$pb-1]->rbar );
+			$pb--;
+		    }
+		    # Find next bar line.
+		    my $nb = $i;
+		    while ( $nb < @$cells ) {
+			last if $cells->[$nb]->rbar
+			  || ( $nb+1 < @$cells && $cells->[$nb+1]->lbar );
+			$nb++;
+		    }
+
+		    # Center between the barlines.
+		    $x -= ( $i-$pb ) * $dx;
+		    $x += ( $nb-$pb+1 ) * $dx/2;
+		    if ( $self->{npp} ) {
+			if (  $c eq "repeat1Bar" ) {
+			    $self->npp_repeat( $x, $y, 1 );
+			}
+			else {
+			    $self->npp_repeat( $x, $y, 2 );
+			}
+		    }
+		    else {
+			$self->textc( $x, ($y-0.3*$musicsize),
+				      $musicglyphs->{$c}, $chordsize, $musicfont );
+		    }
 		}
 
-		# Center between the barlines.
-		$x -= ( $i-$pb ) * $dx;
-		$x += ( $nb-$pb+1 ) * $dx/2;
-		if ( $self->{npp} ) {
-		    $self->npp_repeat1( $x, $y );
-		}
 		else {
-		    $self->textc( $x, ($y-0.3*$musicsize),
-				  $musicglyphs->{$c}, $chordsize, $musicfont );
-		}
-		next;
-	    }
 
-	    if ( $c =~ /^repeat2Bars$/ ) {
+		    # Find next bar line.
+		    my $nb = $i;
+		    while ( $nb < @$cells ) {
+			last if $cells->[$nb]->rbar
+			  || ( $nb+1 < @$cells && $cells->[$nb+1]->lbar );
+			$nb++;
+		    }
 
-		# Find next bar line.
-		my $nb = $i;
-		while ( $nb < @$cells ) {
-		    last if $cells->[$nb]->rbar
-		      || ( $nb+1 < @$cells && $cells->[$nb+1]->lbar );
-		    $nb++;
-		}
-
-		# Overprint next barline.
-		$x += ( $nb-$i+1 ) * $dx;
-		if ( $self->{npp} ) {
-		    $self->npp_repeat2( $x, $y );
-		}
-		else {
-		    $self->textc( $x, ($y-0.3*$musicsize),
-				  $musicglyphs->{$c}, $chordsize, $musicfont );
+		    # Overprint next barline.
+		    $x += ( $nb-$i+1 ) * $dx;
+		    if ( $self->{npp} ) {
+			$self->npp_repeat( $x, $y, 2 );
+		    }
+		    else {
+			$self->textc( $x, ($y-0.3*$musicsize),
+				      $musicglyphs->{$c}, $chordsize, $musicfont );
+		    }
 		}
 		next;
 	    }
@@ -700,6 +745,13 @@ sub make_image {
 	# }}}
     }
 
+    if ( FONTSX && %missing_glyphs ) {
+	printf STDERR ( "Missing glyphs:" );
+	printf STDERR ( " U+%04X", $_)
+	  foreach sort map { abs($_) } keys %missing_glyphs;
+	print STDERR ("\n");
+    }
+
     # Crop excess bottom space.
     if ( $self->{im} && $self->{crop} && $low ) {
 	$self->{im} = $self->{im}->crop( top => 0, height => scale($low) );
@@ -1202,21 +1254,12 @@ sub npp_mark {
 			     tx => $x-55, ty => $y-76 );
 }
 
-sub npp_repeat1 {
-    my ( $self, $x, $y ) = @_;
-    my $r = $self->getimg("root_x");
+sub npp_repeat {
+    my ( $self, $x, $y, $n ) = @_;
+    my $r = $self->getimg("root_" . ("x" x $n));
     my $w = $r->getwidth;
     $self->{im}->rubthrough( src => $r,
-			     tx => $x - $w/2 - 6,
-			     ty => $y + 56 );
-}
-
-sub npp_repeat2 {
-    my ( $self, $x, $y ) = @_;
-    my $r = $self->getimg("root_xx");
-    my $w = $r->getwidth;
-    $self->{im}->rubthrough( src => $r,
-			     tx => $x - $w/2 - 6,
+			     tx => $x - $w/2 - 8,
 			     ty => $y + 56 );
 }
 
