@@ -5,8 +5,8 @@
 # Author          : Johan Vromans
 # Created On      : Fri Jan 15 19:15:00 2016
 # Last Modified By: Johan Vromans
-# Last Modified On: Thu Nov 10 22:03:10 2016
-# Update Count    : 93
+# Last Modified On: Tue Dec 20 20:30:09 2016
+# Update Count    : 104
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -16,7 +16,7 @@ use warnings;
 use FindBin;
 use lib "$FindBin::Bin/../CPAN";
 use lib "$FindBin::Bin/../lib";
-use Data::iRealPro;
+use Data::iRealPro 1.02;
 use Data::iRealPro::Output;
 
 ################ Setup  ################
@@ -39,6 +39,7 @@ main($options);
 sub main {
     my ($options) = @_;
     binmode(STDERR,':utf8');
+    @ARGV = map { glob } @ARGV if $^O eq 'MSWin32';
     Data::iRealPro::Output->new($options)->processfiles(@ARGV);
 }
 
@@ -93,6 +94,8 @@ sub app_setup {
        output		=> undef,
        transpose	=> 0,
        toc		=> undef,
+       neatify		=> 0,
+       catalog		=> "$FindBin::Bin/../res/catalog.xml",
 
        # Development options (not shown with -help).
        debug		=> 0,		# debugging
@@ -127,6 +130,8 @@ sub app_setup {
 	  'npp=s',
 	  'transpose|x=i',
 	  'toc!',
+	  'catalog=s',
+	  'neatify=i',
 
 	  # # Configuration handling.
 	  # 'config=s',
@@ -237,6 +242,11 @@ See the B<--npp> option for details.
 
 See also L<NPP IMAGING>.
 
+If multiple output pages are to be generated you can add a sprintf()
+compliant %d sequence in the output file name. For example, with
+B<--output=img%04d.png> output files will be B<img0001.png>,
+B<img0002.png>, and so on.
+
 =item JSON
 
 This is basically a low-level representation of the contents of the
@@ -268,7 +278,7 @@ B<--dir>).
 
 Specifies the desired output file name.
 
-The file name extension controls the output type.
+The file name extension controls the output type. 
 
 =item B<--select=>I<NN>
 
